
# q-expansion of newform 3325.2.a.b, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3325
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [0], [1], [3], [-1], [-6], [-1], [8], [-2], [-5], [-6], [-1], [7], [10], [2], [-11], [-10], [6], [8], [0], [0], [2], [-1], [-1], [4], [8], [18], [-2], [-10], [2], [-18], [3], [-4], [-7], [4], [-8], [15], [-17], [-21], [-2], [6], [9], [-26], [-21], [-20], [-14], [-12], [-3], [-6], [-11], [-11], [-11], [16], [13], [-19], [-3], [30], [-19], [-16], [-16], [26], [20], [-12], [-6], [-12], [-4], [8], [-33], [10], [2], [12], [-2], [-20], [-16], [-27], [6], [-4], [-40], [-26], [30], [22], [0], [17], [23], [1], [-30], [38], [-24], [41], [30], [-24], [-18], [-41], [9], [-30], [30], [5], [-17], [-45], [38], [13], [-24], [20], [39], [0], [-10], [-20], [-36], [-27], [-45], [-6], [39], [26], [47], [12], [10], [28], [14], [-30], [-25], [-26], [-19], [-42], [-20], [3], [18], [-42], [22], [-14], [39], [18], [-2], [-2], [40], [44], [-54], [-17], [26], [-18], [-4], [-23], [23], [-6], [-30], [24], [-14], [6], [-38], [18], [38], [0], [25], [-40], [46], [-48], [37], [-6], [22], [41], [9], [-42], [-29], [27], [30], [-32], [16], [0], [2], [30], [-10], [-14], [40], [45], [-5], [-4], [28], [-30], [-24], [50], [-9], [21], [-14], [-2], [33], [2], [-26], [32], [42], [-30], [33], [9], [-34], [-39], [-12], [-39], [-29], [-26], [-30], [-54], [-48], [-23], [29], [53], [-44], [-28], [20], [68], [-15], [15], [-47], [-2], [64], [-53], [-56], [26], [56], [1], [-46], [14], [-44], [11], [-32], [-19], [27], [25], [73], [-46], [-21], [-15], [-59], [53], [-18], [46], [-2], [42], [22], [22], [-20], [21], [22], [-42], [60], [-58], [41], [-42], [-54], [-15], [68], [44], [-69], [-56], [69], [53], [4], [-8], [47], [-3], [-38], [10], [-58], [16], [-58], [46], [-18], [21], [22], [-54], [-10], [-3], [-53], [0], [-32], [20], [-77], [60], [-32], [62], [-48], [-60], [47], [16], [80], [9], [-62], [59], [6], [68], [-10], [53], [61], [-18], [4], [-84], [-26], [-62], [63], [-18], [-75], [-8], [0], [26], [13], [-48], [-47], [56], [42], [-78], [17], [-69], [90], [-55], [48], [-48], [29], [-33], [10], [46], [-74], [-69], [31], [-29], [-7], [34], [65], [1], [-62], [21], [48], [-53], [-50], [52], [60], [44], [63], [-78], [48], [-45], [58], [-74], [54], [19], [15], [-46], [2], [30], [44], [-94], [-77], [22], [35], [52], [-21], [73], [62], [-3], [48], [72], [-27], [24], [-50], [36], [-12], [-94], [56], [-8], [24], [10], [36], [93], [35], [48], [-36], [76], [37], [85], [80], [38], [18], [26], [-75], [-53], [-11], [71], [-65], [-43], [-3], [78], [13], [40], [22], [-5], [-27], [46], [-38], [76], [-9], [-85], [20], [54], [62], [-17], [20], [21], [70], [52], [-2], [6], [-28], [16], [57], [21], [30], [99], [63], [-36], [12], [-104], [-12], [84], [14], [-14], [24], [-51], [-74], [63], [-22], [-14], [102], [58]]
